Package: misc3d
Title: Miscellaneous 3D Plots
Version: 0.9-2
Authors@R: c(person("Luke", "Tierney", role = c("aut", "cre"),
                    email = "luke-tierney@uiowa.edu"),
	     person("Dai", "Feng", role="aut"))
Suggests: rgl, tkrplot, MASS, maps, lattice, scatterplot3d
Imports: grDevices, graphics, stats, tcltk
Description: A collection of miscellaneous 3d plots, including
    isosurfaces.
URL: https://gitlab.com/luke-tierney/misc3d
License: GPL
NeedsCompilation: no
Packaged: 2026-01-27 18:01:43 UTC; luke
Author: Luke Tierney [aut, cre],
  Dai Feng [aut]
Maintainer: Luke Tierney <luke-tierney@uiowa.edu>
Repository: CRAN
Date/Publication: 2026-01-28 06:10:33 UTC
