export(contour3d,computeContour3d)
export(image3d)
export(parametric3d)
export(slices3d)
export(drawScene,drawScene.rgl)
export(exportScene)
export(surfaceTriangles)
export(makeTriangles,updateTriangles)
export(scaleTriangles,translateTriangles,transformTriangles)
export(perspLighting,phongLighting)
export(kde3d)
export(pointsTetrahedra)
export(linesTetrahedra)

importFrom("grDevices", "col2rgb", "gray.colors", "heat.colors", "rgb")
importFrom("graphics", "image", "lines", "par", "polygon")
importFrom("stats", "dnorm", "quantile", "runif")
importFrom("tcltk", "tclvalue", "tclVar", "tclvalue<-",
           "tkconfigure", "tkbind", "tkframe",
           "tkgrid", "tklabel", "tkscale", "tktitle<-", 
           "tktoplevel", "tkwinfo")